package com.game.business.controller;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.game.business.domain.AppUser;
import com.game.business.domain.AppUserAgent;
import com.game.business.domain.AppUserCount;
import com.game.business.domain.FinTranRecord;
import com.game.business.dto.*;
import com.game.business.service.IAppUserAgentService;
import com.game.business.service.IAppUserCountService;
import com.game.business.service.IAppUserService;
import com.game.business.service.IFinTranRecordService;
import com.game.business.vo.AppAgentTeamVo;
import com.game.business.vo.AppUserAgentJournalIndexVO;
import com.game.business.vo.AppUserTeamTranserRecordVo;
import com.game.common.core.controller.BaseController;
import com.game.common.core.domain.HttpRet;
import com.game.common.core.domain.HttpRetPageArr;
import com.game.common.core.page.TableDataInfo;
import com.game.common.utils.MessageUtils;
import com.game.common.utils.SecurityUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/business/agent")
@Api(value = "AppGameBettingController", description = "用户代理表接口", tags = {"用户代理表接口"})
public class AppAgentController extends BaseController {

    @Autowired
    private IAppUserCountService appUserCountService;

    @Autowired
    private IAppUserService appUserService;

    @Autowired
    private IAppUserAgentService appUserAgentService;

    @Autowired
    private IFinTranRecordService finTranRecordService;


    /**
     * 首页
     */
    @PostMapping(value = "/index")
    @ApiOperation(value = "用户代理统计", notes = "用户代理统计")
    public HttpRet<AppUserAgentJournalIndexVO> index(@RequestBody AgentIndexDTO agentIndexDto)
    {
        Long userId = SecurityUtils.getUserId();
        if(null != agentIndexDto.getUserId()){
            userId = agentIndexDto.getUserId();
        }
        AppUserAgentJournalIndexVO vo = new AppUserAgentJournalIndexVO();
        Date nowDate = DateUtil.date();
        if(agentIndexDto.getType() == 1){
            nowDate = DateUtil.offsetMonth(nowDate,-1); //上个月
        }
        String formatP = "yyyy-MM-dd";
        Date beginDate = DateUtil.beginOfMonth(nowDate);
        Date endDate = DateUtil.endOfMonth(nowDate);

        List<AppUserCount> appUserCountList = appUserCountService.getAppUserCount(userId,DateUtil.format(beginDate,formatP),DateUtil.format(endDate,formatP));

        if(null != appUserCountList){
            appUserCountList = new ArrayList<>();
        }
        //佣金
        vo.setMyCommission(appUserCountList.stream().mapToDouble(e->e.getLiveCommission() + e.getGameCommission()).sum());
        //团队人数
        vo.setTeamUser(appUserAgentService.countAgent(userId,null,null));
        //团队充值
        vo.setTeamRecharge(appUserCountList.stream().mapToDouble(e->e.getRechargeAmount()).sum());
        //团队投注
        vo.setTeamBetting(appUserAgentService.sumBetting(DateUtil.format(beginDate,"yyyy-MM-dd").concat(" 00:00:00")
                ,DateUtil.format(endDate,"yyyy-MM-dd").concat(" 23:59:59"),userId));
        //团队直播消费
        vo.setTeamLive(appUserCountList.stream().mapToDouble(e->e.getLiveUseAmount()).sum());
        //团队投注赢
        vo.setTeamWin(appUserCountList.stream().mapToDouble(e->e.getGameWinAmount()).sum());
        //团队投注输
        vo.setTeamLose(appUserCountList.stream().mapToDouble(e->e.getGameLoseAmount()).sum());
        //活跃人数
        vo.setActiveUser(appUserCountList.stream().collect(Collectors.collectingAndThen(Collectors.toCollection(()-> new TreeSet<>(Comparator.comparing(f->f.getUserId()))), ArrayList::new)).size());
        //昨日新增
        Date yesterDay = DateUtil.offsetDay(DateUtil.date(),-1);
        vo.setYesterdayUser(appUserAgentService.countAgent(userId,DateUtil.format(yesterDay,"yyyy-MM-dd").concat(" 00:00:00")
                ,DateUtil.format(yesterDay,"yyyy-MM-dd").concat(" 23:59:59")));
        vo.setMonthUser(appUserAgentService.countAgent(userId,DateUtil.format(beginDate,"yyyy-MM-dd").concat(" 00:00:00")
                ,DateUtil.format(endDate,"yyyy-MM-dd").concat(" 23:59:59")));

        return HttpRet.success(MessageUtils.message("11022"),vo);
    }

    /**
     * 修改直播分成、分红保底
     */
    /*@PostMapping(value = "/updateSetting")
    @ApiOperation(value = "修改直播分成、分红保底", notes = "修改直播分成、分红保底")
    public HttpRet<Boolean> updateSetting(@RequestBody AppAgentEditDto editDto)
    {
        return appUserAgentService.updateSetting(editDto);
    }*/

    /**
     * 团队列表
     */
    @PostMapping(value = "/teamList")
    @ApiOperation(value = "团队列表", notes = "团队列表")
    public HttpRetPageArr<AppAgentTeamVo> teamList(@RequestBody AppAgentTeamDto teamDto)
    {
        startPage();
        List<AppAgentTeamVo> list = appUserAgentService.teamList(teamDto);
        return HttpRetPageArr.success(MessageUtils.message("11022"),getDataTable(list));
    }

    /**
     * 团队转账
     */
    @PostMapping(value = "/transfer")
    @ApiOperation(value = "团队转账", notes = "团队转账")
    public HttpRet<Boolean> transfer(@RequestBody @Validated AppUserTeamTranserDto transerDto)
    {
        Long userId = SecurityUtils.getUserId();
       //查询当前用户代理信息
       LambdaQueryWrapper<AppUserAgent> queryWrapper = new LambdaQueryWrapper<>();
       queryWrapper.eq(AppUserAgent::getUserId,userId);
       queryWrapper.eq(AppUserAgent::getAuditStatus,1);
       AppUserAgent appUserAgent = appUserAgentService.getOne(queryWrapper);
       if(null == appUserAgent){
           return HttpRet.fail("转账失败，受益人不是代理");
       }
       if(appUserAgent.getPid().longValue() == 0 || isTwoLevel(appUserAgent.getPid())){
           List<AppUserAgent> appUserAgentList = appUserAgentService.selectAllAgentList(userId,transerDto.getToUserId());
           if(null == appUserAgentList || appUserAgentList.size() < 1){
               return HttpRet.fail("转账失败，只能给团队成员转账");
           }
       }else{
            if(!isTeam(userId,transerDto.getToUserId())){
                return HttpRet.fail("转账失败，只能给下级成员转账");
            }
       }
       return finTranRecordService.transfer(transerDto);
    }

    /**
     * 团队转账记录
     */
    @PostMapping(value = "/transferRecord")
    @ApiOperation(value = "团队转账记录(分页 链接上拼pageSize和pageNum)", notes = "团队转账记录（分页 链接上拼pageSize和pageNum）")
    public HttpRetPageArr<AppUserTeamTranserRecordVo> transferRecord(@RequestBody @Validated AppUserTeamTranserRecordDto transerDto)
    {
        startPage();
        List<FinTranRecord> list = finTranRecordService.transferRecord(transerDto);
        TableDataInfo<FinTranRecord> tableDataInfo = getDataTable(list);
        TableDataInfo resultDataInfo = new TableDataInfo();
        BeanUtils.copyProperties(tableDataInfo,resultDataInfo);
        if(null != tableDataInfo.getRows() && tableDataInfo.getRows().size() > 0){
            List<AppUser> appUsers = null;
            if(0 == transerDto.getType()){
                //转出
                appUsers = appUserService.selectListByIds(
                        tableDataInfo.getRows().stream().map(e->e.getToUid()).collect(Collectors.toList())
                );
            }else{
                //转入
                appUsers = appUserService.selectListByIds(
                        tableDataInfo.getRows().stream().map(e->e.getFromUid()).collect(Collectors.toList())
                );
            }
            Map<String,AppUser> userMap = new HashMap<>();
            if(null != appUsers && appUsers.size() > 0){
                userMap = appUsers.stream().collect(Collectors.toMap(e->{return String.valueOf(e.getUserid());}, Function.identity(),(k1,k2)->k2));
            }
            Map<String, AppUser> finalUserMap = userMap;
            resultDataInfo.setRows(tableDataInfo.getRows().stream().map(e->{
                AppUserTeamTranserRecordVo vo = new AppUserTeamTranserRecordVo();
                vo.setMoney(e.getDiamondCoinChange());
                vo.setTime(e.getCreateTime());
                AppUser appUser = null;
                if(0 == transerDto.getType()){
                    //转出
                    long userId = e.getToUid();
                    appUser = finalUserMap.get(String.valueOf(userId));
                    vo.setUserId(userId);
                    if(null != appUser){
                        vo.setNickName(appUser.getUsername());
                    }
                }else{
                    //转入
                    long userId = e.getFromUid();
                    appUser = finalUserMap.get(String.valueOf(userId));
                    vo.setUserId(userId);
                    if(null != appUser){
                        vo.setNickName(appUser.getUsername());
                    }
                }
                return vo;
            }).collect(Collectors.toList()));
        }else{
            resultDataInfo.setRows(new ArrayList());
        }
        return HttpRetPageArr.success(MessageUtils.message("11022"),resultDataInfo);
    }

    //查询用户是否为下级代理
    private boolean isTeam(Long pid,Long userId){
        //查询当前用户代理信息
        LambdaQueryWrapper<AppUserAgent> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AppUserAgent::getUserId,userId);
        queryWrapper.eq(AppUserAgent::getAuditStatus,1);
        AppUserAgent appUserAgent = appUserAgentService.getOne(queryWrapper);
        if(null != appUserAgent && appUserAgent.getPid().longValue() == pid){
            return true;
        }
        return false;
    }

    /**
     * 查询是否为第二级代理
     * */
    private boolean isTwoLevel(Long userId){
        LambdaQueryWrapper<AppUserAgent> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AppUserAgent::getUserId,userId);
        AppUserAgent appUserAgent = appUserAgentService.getOne(queryWrapper);
        if(appUserAgent.getPid().longValue() == 0){
            return true;
        }
        return false;
    }

}
